using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace CheesyFX
{
    public class Throat : Orifice
    {
        private float nippleStim;
        public override void Init(string name)
        {
            base.Init(name);
            DAZCharacterSelector characterSelector = FillMeUp.atom.GetStorableByID("geometry") as DAZCharacterSelector;
            GenerateDAZMorphsControlUI morphControl = characterSelector.morphsControlUI;
			
            depthReference = FillMeUp.atom.rigidbodies.First(x => x.name == "ThroatTrigger");
            name = "Throat";
            // onEnterActions = orificeManager.bodyManager.audioManager.Stop;
            sensitivity.val = .2f;
            audioSource = depthReference.gameObject.AddComponent<AudioSource>();

            bulgeScale.val = bulgeScale.defaultVal = 2f;
            bulgeDepthScale.val = bulgeDepthScale.defaultVal = 1.1f;
            
            bulgeFunc = x =>
            {
                float val = (3f*penetratorWidth)*bulgeScale.val*Mathf.Min(10f * depth.val * bulgeDepthScale.val, 1f) * Mathf.Pow(2f, -x * x *1e-4f* Mathf.Pow(depth.val*bulgeDepthScale.val, -4));
                return val;
            };
        }

        public override void FixedUpdate()
        {
            base.FixedUpdate();
            var absSpeed = Mathf.Abs(speed.val);
            if(!audioSource.isPlaying && absSpeed > .05f && depth.val > .05f)
            {
                var clip = FillMeUp.bjLibrary.GetRandomClip();
                audioSource.clip = clip;
                audioSource.volume = 10f * absSpeed * (1f + depth.val)*(1f + depth.val);
                audioSource.Play();
            }

            if (depth.val > .1f)
            {
                nippleStim = (depth.val - .1f) * Time.fixedDeltaTime * 6f;
                NippleManager.nippleDrivers[0].Stimulate(nippleStim);
                NippleManager.nippleDrivers[1].Stimulate(nippleStim);
            }
        }
        
        public override void CreateBulgeSettingsUI()
        {
            bulgeScale.CreateUI(FillMeUp.singleton, UIElements:UIElements);
            bulgeDepthScale.CreateUI(FillMeUp.singleton, UIElements:UIElements);
        }
        
        public override void RegisterCollision(Collider collider)
        {
            GetPenetrator(collider);
            depthMeter.transform.parent = penetrator.tip;
            depthMeter.transform.localPosition = Vector3.zero;
            penetratorWidth = penetrator.width * stretchScale.val;
            isPenetrated = true;
            penetratingAtom = penetrator.atom;
        }
    }
}